import json
from importlib import import_module
from unittest.mock import patch

from django.contrib.auth.models import AnonymousUser
from django.http import Http404
from django.test import RequestFactory
from django.test import TestCase

from blog.models import Post
from hitcount.conf import settings
from hitcount.utils import get_hitcount_model
from hitcount.views import HitCountDetailView
from hitcount.views import HitCountJSONView

HitCount = get_hitcount_model()


class BaseHitCountViewTest(TestCase):
    def setUp(self):
        self.post = Post.objects.create(title="my title", content="my text")
        self.hit_count = HitCount.objects.create(content_object=self.post)
        self.factory = RequestFactory()
        self.request_post = self.factory.post(
            "/",
            {"hitcountPK": self.hit_count.pk},
            REMOTE_ADDR="127.0.0.1",
            HTTP_USER_AGENT="my_clever_agent",
            HTTP_X_REQUESTED_WITH="XMLHttpRequest",
        )
        self.request_get = self.factory.get(
            "/", REMOTE_ADDR="127.0.0.1", HTTP_USER_AGENT="my_clever_agent", HTTP_X_REQUESTED_WITH="XMLHttpRequest"
        )

        self.engine = import_module(settings.SESSION_ENGINE)
        self.store = self.engine.SessionStore()
        self.store.save()
        self.request_post.session = self.store
        self.request_post.user = AnonymousUser()
        self.request_get.session = self.store
        self.request_get.user = AnonymousUser()


class TestHitCountJSONView(BaseHitCountViewTest):
    def test_require_ajax(self):
        non_ajax_request = self.factory.get("/")
        with self.assertRaises(Http404):
            HitCountJSONView.as_view()(non_ajax_request)

    def test_require_post_only(self):
        """
        Test require POST request or raise 404
        """
        non_ajax_request = self.factory.get("/", HTTP_X_REQUESTED_WITH="XMLHttpRequest")
        non_ajax_request.session = self.store
        response = HitCountJSONView.as_view()(non_ajax_request)
        json_response = json.loads(response.content.decode())
        json_expects = json.loads('{"error_message": "Hits counted via POST only.", "success": false}')

        self.assertEqual(json_response, json_expects)

    def test_count_hit(self):
        """
        Test a valid request.
        """
        response = HitCountJSONView.as_view()(self.request_post)

        self.assertEqual(response.content, b'{"hit_counted": true, "hit_message": "Hit counted: session key"}')

    def test_count_hit_invalid_hitcount_pk(self):
        """
        Test a valid request with an invalid hitcount pk.
        """
        wrong_pk_request = self.factory.post(
            "/",
            {"hitcountPK": 15},
            REMOTE_ADDR="127.0.0.1",
            HTTP_USER_AGENT="my_clever_agent",
            HTTP_X_REQUESTED_WITH="XMLHttpRequest",
        )
        wrong_pk_request.session = self.store

        response = HitCountJSONView.as_view()(wrong_pk_request)

        self.assertEqual(response.content, b"HitCount object_pk not present.")


class TestHitCountDetailView(BaseHitCountViewTest):
    def test_count_hit(self):
        """
        Test a valid request.
        """
        view = HitCountDetailView.as_view(model=Post)

        response = view(self.request_get, pk=self.post.pk)

        self.assertEqual(response.context_data["hitcount"]["pk"], self.hit_count.pk)
        self.assertEqual(response.context_data["hitcount"]["total_hits"], 0)

    def test_count_hit_incremented(self):
        """
        Increment a hit and then get the response.
        """
        view = HitCountDetailView.as_view(model=Post, count_hit=True)

        response = view(self.request_get, pk=self.post.pk)

        self.assertEqual(response.context_data["hitcount"]["total_hits"], 1)
        self.assertEqual(response.context_data["hitcount"]["pk"], self.hit_count.pk)

    @patch.object(settings, "HITCOUNT_HITS_PER_SESSION_LIMIT", 1)
    def test_count_hit_incremented_only_once(self):
        """
        Increment a hit and then get the response.
        """
        view = HitCountDetailView.as_view(model=Post, count_hit=True)

        response = view(self.request_get, pk=self.post.pk)

        self.assertEqual(response.context_data["hitcount"]["total_hits"], 1)
        self.assertEqual(response.context_data["hitcount"]["pk"], self.hit_count.pk)
        view = HitCountDetailView.as_view(model=Post, count_hit=True)

        response = view(self.request_get, pk=self.post.pk)

        self.assertEqual(response.context_data["hitcount"]["total_hits"], 1)
        self.assertEqual(response.context_data["hitcount"]["pk"], self.hit_count.pk)



from django.db import models


class BlockedIPManager(models.Manager):
    def is_blocked(self, ip=None):
        if not ip:
            return False
        return self.filter(ip__exact=ip).exists()


class BlockedUserAgentManager(models.Manager):
    def is_blocked(self, user_agent=None):
        if not user_agent:
            return False

        return self.filter(user_agent__exact=user_agent).exists()



from datetime import timedelta
from importlib import import_module
from unittest.mock import patch

from django.contrib.auth.models import Group
from django.contrib.auth.models import User
from django.contrib.sessions.backends.db import SessionStore
from django.utils import timezone

from hitcount.conf import settings
from hitcount.mixins import HitCountViewMixin
from hitcount.models import BlockedIP
from hitcount.models import BlockedUserAgent
from hitcount.models import Hit
from hitcount.models import HitCount
from tests.test_views import BaseHitCountViewTest


@patch.object(settings, "HITCOUNT_USE_IP", True)
class TestHitCountViewMixin(BaseHitCountViewTest):
    def test_when_use_ip_is_disabled(self):
        with patch.object(settings, "HITCOUNT_USE_IP", False):
            response = HitCountViewMixin.hit_count(self.request_post, self.hit_count)

        self.assertIs(response.hit_counted, True)
        self.assertEqual(response.hit_message, "Hit counted: session key")

        # test database
        hit = Hit.objects.last()
        self.assertIsNone(hit.ip)

    def test_session_key_is_not_present(self):
        session = SessionStore(session_key=None)

        self.request_post.session = session

        response = HitCountViewMixin.hit_count(self.request_post, self.hit_count)

        self.assertIs(response.hit_counted, True)
        self.assertEqual(response.hit_message, "Hit counted: session key")
        # test database
        hit = Hit.objects.last()
        self.assertIsNotNone(hit.session)

    def test_anonymous_user_hit(self):
        response = HitCountViewMixin.hit_count(self.request_post, self.hit_count)

        self.assertIs(response.hit_counted, True)
        self.assertEqual(response.hit_message, "Hit counted: session key")

    @patch.object(settings, "HITCOUNT_HITS_PER_SESSION_LIMIT", 1)
    def test_multiple_anonymous_user_hit_not_counted_with_session_limit(self):
        response = HitCountViewMixin.hit_count(self.request_post, self.hit_count)
        self.assertIs(response.hit_counted, True)
        self.assertEqual(response.hit_message, "Hit counted: session key")

        response = HitCountViewMixin.hit_count(self.request_post, self.hit_count)
        self.assertIs(response.hit_counted, False)
        self.assertEqual(response.hit_message, "Not counted: hits per session limit reached.")

    def test_multiple_anonymous_user_hit_counted_after_filter_active(self):
        # create a Hit ten days ago
        created = timezone.now() - timedelta(days=10)
        with patch("django.utils.timezone.now") as mock_now:
            mock_now.return_value = created

            response = HitCountViewMixin.hit_count(self.request_post, self.hit_count)

        self.assertIs(response.hit_counted, True)
        self.assertEqual(response.hit_message, "Hit counted: session key")

        # test a Hit today, within the filter time
        response = HitCountViewMixin.hit_count(self.request_post, self.hit_count)
        self.assertIs(response.hit_counted, True)
        self.assertEqual(response.hit_message, "Hit counted: session key")

    def test_registered_user_hit(self):
        self.request_post.user = User.objects.create_user("john", "lennon@thebeatles.com", "johnpassword")
        response = HitCountViewMixin.hit_count(self.request_post, self.hit_count)

        self.assertIs(response.hit_counted, True)
        self.assertEqual(response.hit_message, "Hit counted: user authentication")

    @patch.object(settings, "HITCOUNT_HITS_PER_SESSION_LIMIT", 1)
    def test_registered_user_hit_with_session_limit(self):
        self.request_post.user = User.objects.create_user("john", "lennon@thebeatles.com", "johnpassword")

        response = HitCountViewMixin.hit_count(self.request_post, self.hit_count)
        self.assertIs(response.hit_counted, True)
        self.assertEqual(response.hit_message, "Hit counted: user authentication")

        response = HitCountViewMixin.hit_count(self.request_post, self.hit_count)
        self.assertIs(response.hit_counted, False)
        self.assertEqual(response.hit_message, "Not counted: hits per session limit reached.")

    def test_registered_user_hit_counted_after_filter_active(self):
        self.request_post.user = User.objects.create_user("john", "lennon@thebeatles.com", "johnpassword")

        # create a Hit ten days ago
        created = timezone.now() - timedelta(days=10)
        with patch("django.utils.timezone.now") as mock_now:
            mock_now.return_value = created

            response = HitCountViewMixin.hit_count(self.request_post, self.hit_count)

        self.assertIs(response.hit_counted, True)
        self.assertEqual(response.hit_message, "Hit counted: user authentication")

        # test a Hit today, within the filter time
        response = HitCountViewMixin.hit_count(self.request_post, self.hit_count)
        self.assertIs(response.hit_counted, True)
        self.assertEqual(response.hit_message, "Hit counted: user authentication")

    def test_hits_per_ip_limit(self):
        """
        Test `HITCOUNT_HITS_PER_IP_LIMIT` setting.  Should allow multiple hits
        from the same IP until the limit is reached from that IP.
        """
        with patch.object(settings, "HITCOUNT_HITS_PER_IP_LIMIT", 2):
            responses = []
            for _ in range(3):
                # need a new session key each time.
                engine = import_module(settings.SESSION_ENGINE)
                store = engine.SessionStore()
                store.save()
                self.request_post.session = store
                responses.append(HitCountViewMixin.hit_count(self.request_post, self.hit_count))

        self.assertIs(responses[0].hit_counted, True)
        self.assertEqual(responses[0].hit_message, "Hit counted: session key")
        self.assertIs(responses[1].hit_counted, True)
        self.assertEqual(responses[1].hit_message, "Hit counted: session key")
        self.assertIs(responses[2].hit_counted, False)
        self.assertEqual(responses[2].hit_message, "Not counted: hits per IP address limit reached")
        # test database changes
        hit_count = HitCount.objects.get(pk=self.hit_count.pk)

        self.assertEqual(hit_count.hits, 2)

    @patch.object(settings, "HITCOUNT_HITS_PER_SESSION_LIMIT", 1)
    def test_hits_per_session_limit_for_anonymous_user(self):
        response = HitCountViewMixin.hit_count(self.request_post, self.hit_count)
        self.assertIs(response.hit_counted, True)
        self.assertEqual(response.hit_message, "Hit counted: session key")

        response = HitCountViewMixin.hit_count(self.request_post, self.hit_count)
        self.assertIs(response.hit_counted, False)
        self.assertEqual(response.hit_message, "Not counted: hits per session limit reached.")

    @patch.object(settings, "HITCOUNT_HITS_PER_SESSION_LIMIT", 1)
    def test_hits_per_session_limit_for_authenticated_user(self):
        self.request_post.user = User.objects.create_user("john", "lennon@thebeatles.com", "johnpassword")

        response = HitCountViewMixin.hit_count(self.request_post, self.hit_count)
        self.assertIs(response.hit_counted, True)
        self.assertEqual(response.hit_message, "Hit counted: user authentication")

        response = HitCountViewMixin.hit_count(self.request_post, self.hit_count)
        self.assertIs(response.hit_counted, False)
        self.assertEqual(response.hit_message, "Not counted: hits per session limit reached.")

    @patch.object(settings, "HITCOUNT_EXCLUDE_USER_GROUP", ("Admin",))
    def test_excluded_user_group_not_counted(self):
        self.request_post.user = User.objects.create_user("john", "lennon@thebeatles.com", "johnpassword")
        group = Group.objects.create(name="Admin")
        group.user_set.add(self.request_post.user)

        response = HitCountViewMixin.hit_count(self.request_post, self.hit_count)

        self.assertIs(response.hit_counted, False)
        self.assertEqual(response.hit_message, "Not counted: user group has been excluded")

    @patch.object(settings, "HITCOUNT_EXCLUDE_USER_GROUP", ("Admin",))
    def test_not_excluded_user_group_counted(self):
        self.request_post.user = User.objects.create_user("john", "lennon@thebeatles.com", "johnpassword")

        response = HitCountViewMixin.hit_count(self.request_post, self.hit_count)

        self.assertIs(response.hit_counted, True)
        self.assertEqual(response.hit_message, "Hit counted: user authentication")

    def test_blocked_ip(self):
        BlockedIP.objects.create(ip="127.0.0.1")

        response = HitCountViewMixin.hit_count(self.request_post, self.hit_count)
        self.assertIs(response.hit_counted, False)
        self.assertEqual(response.hit_message, "Not counted: user IP has been blocked")

    def test_blocked_user_agent(self):
        BlockedUserAgent.objects.create(user_agent="my_clever_agent")

        response = HitCountViewMixin.hit_count(self.request_post, self.hit_count)
        self.assertIs(response.hit_counted, False)
        self.assertEqual(response.hit_message, "Not counted: user agent has been blocked")



from hitcount.managers.blockers import BlockedIPManager
from hitcount.managers.blockers import BlockedUserAgentManager
from hitcount.managers.hits import HitCountManager
from hitcount.managers.hits import HitManager


__all__ = (
    "HitCountManager",
    "HitManager",
    "BlockedIPManager",
    "BlockedUserAgentManager",
)



from datetime import timedelta
from importlib import import_module
from unittest.mock import patch

from django.test import RequestFactory
from django.test import TestCase
from django.utils import timezone

from blog.models import Post
from hitcount.conf import settings
from hitcount.models import Hit
from hitcount.models import HitCount


@patch.object(settings, "HITCOUNT_USE_IP", True)
class TestHitManager(TestCase):
    def setUp(self):
        post = Post.objects.create(title="my title", content="my text")
        self.hitcount = HitCount.objects.create(content_object=post)
        self.hit = Hit.objects.create(hitcount=self.hitcount)

    @patch.object(settings, "HITCOUNT_KEEP_HIT_ACTIVE", {"days": 7})
    def test_filter_active(self):
        """
        Test for "active" Hits.  Out of ten, should have seven remaining.

        """
        hit_count = HitCount.objects.last()

        # add 9 more Hits
        for x in range(9):
            created = timezone.now() - timedelta(days=x + 1)
            with patch("django.utils.timezone.now") as mock_now:
                mock_now.return_value = created

                Hit.objects.create(hitcount=hit_count)

        self.assertEqual(Hit.objects.all().count(), 10)
        self.assertEqual(Hit.objects.filter_active().count(), 7)

    def test_has_limit_reached_by_ip(self):
        ip = "127.0.0.1"
        Hit.objects.create(hitcount=self.hitcount, ip="127.0.0.1")

        # all hits are counted.
        with patch.object(settings, "HITCOUNT_HITS_PER_IP_LIMIT", 0):
            self.assertIs(Hit.objects.has_limit_reached_by_ip(), False)

            Hit.objects.create(hitcount=self.hitcount, ip=ip)

            self.assertIs(Hit.objects.has_limit_reached_by_ip(ip), False)

        with patch.object(settings, "HITCOUNT_HITS_PER_IP_LIMIT", 2):
            self.assertIs(Hit.objects.has_limit_reached_by_ip(ip), True)

    def test_has_limit_reached_by_session(self):
        request = RequestFactory()
        engine = import_module(settings.SESSION_ENGINE)
        session = engine.SessionStore()
        session.save()
        request.session = session
        session_key = request.session.session_key

        Hit.objects.create(hitcount=self.hitcount, session=session_key)

        # all hits are counted.
        with patch.object(settings, "HITCOUNT_HITS_PER_SESSION_LIMIT", 0):
            self.assertIs(Hit.objects.has_limit_reached_by_session(session_key, self.hitcount), False)

            Hit.objects.create(hitcount=self.hitcount, session=session_key)

            self.assertIs(Hit.objects.has_limit_reached_by_session(session_key, self.hitcount), False)

        with patch.object(settings, "HITCOUNT_HITS_PER_SESSION_LIMIT", 2):
            self.assertIs(Hit.objects.has_limit_reached_by_session(session_key, self.hitcount), True)


class TestHitCountManager(TestCase):
    def setUp(self):
        self.post = Post.objects.create(title="my title", content="my test")

    def test_get_for_object(self):
        post2 = Post.objects.create(title="my title2", content="my text")
        hit_count = HitCount.objects.create(content_object=self.post)
        hit_count2 = HitCount.objects.create(content_object=post2)

        self.assertEqual(HitCount.objects.get_for_object(self.post), hit_count)
        self.assertEqual(HitCount.objects.get_for_object(post2), hit_count2)

    def test_generic_relation(self):
        """
        Test generic relation back to HitCount from a model.

        """
        hit_count = HitCount.objects.create(content_object=self.post)
        hit_count.increase()

        self.assertEqual(HitCount.objects.get_for_object(self.post).hits, self.post.hit_count_generic.get().hits)



from datetime import timedelta
from unittest.mock import patch

from django.test import TestCase
from django.utils import timezone

from blog.models import Post
from hitcount.models import Hit
from hitcount.utils import get_hitcount_model


HitCount = get_hitcount_model()


class TestHitModel(TestCase):
    def setUp(self):
        post = Post.objects.create(title="my title", content="my text")
        hit_count = HitCount.objects.create(content_object=post)
        self.hit = Hit.objects.create(hitcount=hit_count)

    def test_string_representation(self):
        self.assertEqual(str(self.hit), f"Hit: {self.hit.pk}")

    def test_save_does_not_increases_hitcount_for_already_created_obj(self):
        hit_count = HitCount.objects.last()
        init_count = hit_count.hits

        self.hit.save()
        hit_count.refresh_from_db()

        self.assertEqual(hit_count.hits, init_count)

    def test_save_increases_hitcount_for_newly_created_obj(self):
        hit_count = HitCount.objects.last()
        init_count = hit_count.hits

        Hit.objects.create(hitcount=hit_count)
        hit_count.refresh_from_db()

        self.assertEqual(hit_count.hits, init_count + 1)

    def test_hit_count_increase(self):
        """
        Testing if Hit creation triggers increase of associated HitCount

        """
        hit_count = HitCount.objects.last()

        self.assertEqual(hit_count.hits, 1)

        Hit.objects.create(hitcount=hit_count)

        hit_count.refresh_from_db()

        self.assertEqual(hit_count.hits, 2)

    def test_hit_delete(self):
        """
        Testing if Hit deletion triggers decrease of associated HitCount

        """
        self.hit.delete()

        hit_count = HitCount.objects.last()

        self.assertEqual(hit_count.hits, 0)

    def test_hit_delete_save_hitcount(self):
        """
        Testing if Hit deletion with `save_hitcount` flag preserves
        the associated HitCount

        """
        self.hit.delete(save_hitcount=True)

        hit_count = HitCount.objects.last()

        self.assertEqual(hit_count.hits, 1)


class TestHitCountModel(TestCase):
    def setUp(self):
        self.post = Post.objects.create(title="my title", content="my text")

    def test_string_representation(self):
        post = Post.objects.create(title="my title", content="my text")

        hit_count = HitCount(content_object=post)

        self.assertEqual(str(hit_count), "Post title: my title")

    def test_increase(self):
        hit_count = HitCount.objects.create(content_object=self.post)
        hit_count.increase()

        hit_count.refresh_from_db()

        self.assertEqual(hit_count.hits, 1)

    def test_decrease(self):
        hit_count = HitCount.objects.create(hits=1, content_object=self.post)
        hit_count.decrease()

        hit_count.refresh_from_db()

        self.assertEqual(hit_count.hits, 0)

    def test_hits_in_last_assert_error_when_no_args(self):
        hit_count = HitCount.objects.create(content_object=self.post)
        with self.assertRaises(AssertionError):
            hit_count.hits_in_last()

    def test_hits_in_last(self):
        hit_count = HitCount.objects.create(content_object=self.post)

        for x in range(10):
            created = timezone.now() - timedelta(days=x * 5)
            with patch("django.utils.timezone.now") as mock_now:
                mock_now.return_value = created

                Hit.objects.create(hitcount=hit_count)

        hit_count = HitCount.objects.last()

        self.assertEqual(hit_count.hits_in_last(days=30), 6)

    def test_mixin(self):
        hit_count = HitCount.objects.create(content_object=self.post)
        hit_count.increase()

        self.assertEqual(HitCount.objects.get_for_object(self.post).hits, self.post.hit_count.hits)

    def test_mixing_hits_in_last(self):
        """
        Test HitCountMixin `hits_in_last` function.

        """
        hit_count = HitCount.objects.create(content_object=self.post)

        for x in range(10):
            created = timezone.now() - timedelta(days=x * 5)
            with patch("django.utils.timezone.now") as mock_now:
                mock_now.return_value = created

                Hit.objects.create(hitcount=hit_count)

        self.assertEqual(self.post.hit_count.hits_in_last(days=30), 6)

    def test_on_delete_cascade(self):
        """
        Test on_delete=models.CASCADE to ensure that a deleted hitcount
        also removes the related Hits.

        """
        hit_count = HitCount.objects.create(content_object=self.post)

        for _ in range(10):
            Hit.objects.create(hitcount=hit_count)

        self.assertEqual(Hit.objects.all().count(), 10)
        hit_count.delete()

        self.assertEqual(HitCount.objects.all().count(), 0)
        self.assertEqual(Hit.objects.all().count(), 0)



from django.contrib.contenttypes.fields import GenericRelation
from django.db import models

from hitcount.conf import settings as hitcount_settings
from hitcount.mixins import HitCountModelMixin


class Post(models.Model, HitCountModelMixin):
    title = models.CharField(max_length=200)
    content = models.TextField()
    hit_count_generic = GenericRelation(
        hitcount_settings.HITCOUNT_HITCOUNT_MODEL,
        object_id_field="object_pk",
        related_query_name="hit_count_generic_relation",
    )

    def __str__(self):
        return "Post title: %s" % self.title



import pytest
from django.contrib.staticfiles.testing import StaticLiveServerTestCase
from django.urls import reverse
from selenium import webdriver
from selenium.webdriver.common.by import By
from selenium.webdriver.firefox.options import Options
from selenium.webdriver.support import expected_conditions as EC
from selenium.webdriver.support.ui import WebDriverWait


@pytest.mark.slow
class TestUpdateHitCount(StaticLiveServerTestCase):
    def setUp(self):
        options = Options()
        options.add_argument("-headless")
        self.selenium = webdriver.Firefox()
        self.delay = 10

    def tearDown(self):
        self.selenium.quit()

    def test_ajax_hit(self):
        url = reverse("ajax", args=[1])
        self.selenium.get("%s%s" % (self.live_server_url, url))

        wait = WebDriverWait(self.selenium, self.delay)
        response = wait.until(EC.text_to_be_present_in_element((By.ID, "hit-counted-value"), "true"))

        self.assertIs(response, True)



from blog.models import Post
from hitcount.conf import settings
from hitcount.models import HitCount
from hitcount.utils import _get_model_from_string
from hitcount.utils import get_hitcount_model
from hitcount.utils import get_ip


class TestGetIP:
    def test_x_forwared_not_set(self, rf):
        ip = get_ip(rf.get("/"))

        assert ip == "127.0.0.1"

    def test_x_forwarded_set(self, rf):
        rf.defaults["HTTP_X_FORWARDED_FOR"] = "203.0.113.195"

        ip = get_ip(rf.get("/"))

        assert ip == "203.0.113.195"

    def test_neither_remote_addr_nor_x_forwarded_set(self, rf):
        rf.defaults["REMOTE_ADDR"] = ""

        ip = get_ip(rf.get("/"))

        assert ip == ""

    def test_bogus_ip_format(self, rf):
        rf.defaults["REMOTE_ADDR"] = "22.A2.22.22"

        ip = get_ip(rf.get("/"))

        assert ip == "10.0.0.1"


class TestGetModelFromString:
    def test(self):
        assert _get_model_from_string("hitcount.HitCount") == HitCount


class TestGetHitCountModel:
    def test(self, monkeypatch):
        monkeypatch.setattr(settings, "HITCOUNT_HITCOUNT_MODEL", "blog.Post")

        assert get_hitcount_model() == Post



from datetime import timedelta
from io import StringIO
from unittest.mock import patch

from django.core.management import call_command
from django.test import TestCase
from django.utils import timezone

from blog.models import Post
from hitcount.conf import settings
from hitcount.models import Hit
from hitcount.utils import get_hitcount_model


HitCount = get_hitcount_model()


@patch.object(settings, "HITCOUNT_KEEP_HIT_IN_DATABASE", {"days": 30})
class TestHitCountCleanUp(TestCase):
    COMMAND_NAME = "hitcount_cleanup"

    def setUp(self):
        post = Post.objects.create(title="hi", content="some text")
        hit_count = HitCount.objects.create(content_object=post)

        for x in range(10):
            created = timezone.now() - timedelta(days=x * 5)
            with patch("django.utils.timezone.now") as mock_now:
                mock_now.return_value = created

                Hit.objects.create(hitcount=hit_count)

    def test_remove_expired_hits(self):
        """There should be only 6 items remaining after cleanup."""
        call_command(self.COMMAND_NAME)

        self.assertEqual(Hit.objects.all().count(), 6)

    def test_preserve_hitcount(self):
        """Removing Hits should not decrease the total HitCount."""
        hit_count = HitCount.objects.last()

        call_command(self.COMMAND_NAME)

        self.assertEqual(hit_count.hits, 10)

    def test_standard_output(self):
        out = StringIO()

        call_command(self.COMMAND_NAME, stdout=out)

        self.assertIn("Successfully removed 4 Hits", out.getvalue())



from hitcount.models.blockers import BlockedIP
from hitcount.models.blockers import BlockedUserAgent
from hitcount.models.hits import Hit
from hitcount.models.hits import HitCount
from hitcount.models.hits import HitCountBase


__all__ = (
    "HitCountBase",
    "HitCount",
    "Hit",
    "BlockedIP",
    "BlockedUserAgent",
)



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.contrib import admin
from django.core.exceptions import PermissionDenied
from django.utils.translation import gettext_lazy as _

from hitcount.conf import settings
from hitcount.models import BlockedIP
from hitcount.models import BlockedUserAgent
from hitcount.models import Hit
from hitcount.utils import _get_model_from_string


class HitAdmin(admin.ModelAdmin):
    list_display = ("created", "user", "ip", "user_agent", "hitcount")
    search_fields = ("ip", "user_agent")
    date_hierarchy = "created"
    actions = [
        "block_ips",
        "block_user_agents",
        "block_and_delete_ips",
        "block_and_delete_user_agents",
        "delete_queryset",
    ]

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.list_display_links = None

    def has_add_permission(self, request):
        return False

    def get_actions(self, request):
        actions = super().get_actions(request)
        if "delete_selected" in actions:
            del actions["delete_selected"]
        return actions

    def block_ips(self, request, queryset):
        for obj in queryset:
            BlockedIP.objects.get_or_create(ip=obj.ip)

        msg = _("Successfully blockeded %d IPs") % queryset.count()
        self.message_user(request, msg)

    block_ips.short_description = _("Block selected IP addresses")

    def block_user_agents(self, request, queryset):
        for obj in queryset:
            BlockedUserAgent.objects.get_or_create(user_agent=obj.user_agent)

        msg = _("Successfully blocked %d User Agents") % queryset.count()
        self.message_user(request, msg)

    block_user_agents.short_description = _("Add selected User Agents to blocked")

    def block_and_delete_ips(self, request, queryset):
        self.block_ips(request, queryset)
        self.delete_queryset(request, queryset)

    block_and_delete_ips.short_description = _("Delete selected hits and block related IP addresses")

    def block_and_delete_user_agents(self, request, queryset):
        self.block_user_agents(request, queryset)
        self.delete_queryset(request, queryset)

    block_and_delete_user_agents.short_description = _("Delete selected hits and block related User Agents")

    def delete_queryset(self, request, queryset):
        if not self.has_delete_permission(request):
            raise PermissionDenied
        else:
            if queryset.count() == 1:
                msg = "1 hit was"
            else:
                msg = "%s hits were" % queryset.count()

            for obj in queryset.iterator():
                obj.delete()  # calling it this way to get custom delete() method

            self.message_user(request, "%s successfully deleted." % msg)

    delete_queryset.short_description = _("Delete selected hits")


admin.site.register(Hit, HitAdmin)


class HitCountAdmin(admin.ModelAdmin):
    list_display = ("content_object", "hits", "modified")
    fields = ("hits",)

    def has_add_permission(self, request):
        return False


admin.site.register(_get_model_from_string(settings.HITCOUNT_HITCOUNT_MODEL), HitCountAdmin)


class BlockedIPAdmin(admin.ModelAdmin):
    pass


admin.site.register(BlockedIP, BlockedIPAdmin)


class BlockedUserAgentAdmin(admin.ModelAdmin):
    pass


admin.site.register(BlockedUserAgent, BlockedUserAgentAdmin)



from django.apps import AppConfig
from django.utils.translation import gettext_lazy as _


class HitCountAppConfig(AppConfig):
    name = "hitcount"
    verbose_name = _("hitcount")

    def ready(self):
        import hitcount.signals  # noqa: F401



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.dispatch import receiver
from django.dispatch import Signal


delete_hit_count = Signal()


@receiver(delete_hit_count)
def delete_hit_count_handler(sender, instance, *, save_hitcount=False, **kwargs):
    """
    Custom callback for the Hit.delete() method.

    Hit.delete(): removes the hit from the associated HitCount object.
    Hit.delete(save_hitcount=True): preserves the hit for the associated
    HitCount object.
    """
    if not save_hitcount:
        instance.hitcount.decrease()



from django.apps import apps

try:
    from django.utils.regex_helper import _lazy_re_compile
except ImportError:
    from django.core.validators import _lazy_re_compile

from hitcount.conf import settings

# this is not intended to be an all-knowing IP address regex
IP_RE = _lazy_re_compile(r"\d{1,3}\.\d{1,3}\.\d{1,3}\.\d{1,3}")


def get_ip(request):
    """
    Retrieves the remote IP address from the request data.  If the user is
    behind a proxy, they may have a comma-separated list of IP addresses, so
    we need to account for that.  In such a case, only the first IP in the
    list will be retrieved.  Also, some hosts that use a proxy will put the
    REMOTE_ADDR into HTTP_X_FORWARDED_FOR.  This will handle pulling back the
    IP from the proper place.

    **NOTE** This function was taken from django-tracking (MIT LICENSE)
             http://code.google.com/p/django-tracking/

    It has now been modified a bit.
    """
    # if neither header contain a value, just use local loopback
    probable_ip_address = request.META.get("HTTP_X_FORWARDED_FOR", request.META.get("REMOTE_ADDR", "127.0.0.1"))
    if probable_ip_address:
        # make sure we have one and only one IP
        match = IP_RE.match(probable_ip_address)
        if match:
            ip_address = match.group(0)
        else:
            # no IP, probably from some dirty proxy or other device
            # throw in some bogus IP
            ip_address = "10.0.0.1"
        return ip_address

    return probable_ip_address


def _get_model_from_string(model_path):
    app_name, model_name = model_path.rsplit(".", 1)
    return apps.get_model(app_name, model_name)


def get_hitcount_model():
    return _get_model_from_string(settings.HITCOUNT_HITCOUNT_MODEL)



from django.test import SimpleTestCase

from hitcount.models import BlockedIP
from hitcount.models import BlockedUserAgent


class TestBlockedUserAgent(SimpleTestCase):
    def test_string_representation(self):
        ua_text = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_9_4)"

        ua = BlockedUserAgent(user_agent=ua_text)

        self.assertEqual(str(ua), ua_text)


class TestBlockedIPModel(SimpleTestCase):
    def test_string_representation(self):
        ip = BlockedIP(ip="127.0.0.1")

        self.assertEqual(str(ip), "127.0.0.1")



from django.db import models
from django.utils.translation import gettext_lazy as _

from hitcount.managers import BlockedIPManager
from hitcount.managers import BlockedUserAgentManager


class BlockedIP(models.Model):
    ip = models.CharField(max_length=40, unique=True)
    objects = BlockedIPManager()

    class Meta:
        verbose_name = _("Blocked IP")
        verbose_name_plural = _("Blocked IPs")

    def __str__(self):
        return self.ip


class BlockedUserAgent(models.Model):
    user_agent = models.CharField(max_length=255, unique=True)
    objects = BlockedUserAgentManager()

    class Meta:
        verbose_name = _("Blocked User Agent")
        verbose_name_plural = _("Blocked User Agents")

    def __str__(self):
        return "%s" % self.user_agent



from django.contrib.admin.sites import AdminSite
from django.contrib.auth.models import AnonymousUser
from django.contrib.auth.models import User
from django.contrib.messages.storage.fallback import FallbackStorage
from django.core.exceptions import PermissionDenied
from django.test import RequestFactory
from django.test import SimpleTestCase
from django.test import TestCase
from django.urls import reverse

from blog.models import Post
from hitcount.admin import HitAdmin
from hitcount.admin import HitCountAdmin
from hitcount.models import BlockedIP
from hitcount.models import BlockedUserAgent
from hitcount.models import Hit
from hitcount.utils import get_hitcount_model


HitCount = get_hitcount_model()


class TestHitCountAdmin(SimpleTestCase):
    def setUp(self):
        self.factory = RequestFactory()
        self.admin = HitCountAdmin(HitCount, AdminSite())

    def test_has_add_permission_is_always_false(self):
        self.assertIs(self.admin.has_add_permission(self.factory), False)


class TestHitAdmin(TestCase):
    def setUp(self):
        self.factory = RequestFactory()
        self.admin = HitAdmin(Hit, AdminSite())
        self.request = self.factory.get(reverse("admin:hitcount_hit_changelist"))

        # https://code.djangoproject.com/ticket/17971
        self.request.session = "session"
        messages = FallbackStorage(self.request)
        self.request._messages = messages

        post = Post.objects.create(title="my title", content="my text")
        hit_count = HitCount.objects.create(content_object=post)

        for x in range(10):
            Hit.objects.create(hitcount=hit_count, ip="127.0.0.%s" % x, user_agent="agent_%s" % x)

    def test_has_add_permission(self):
        self.assertIs(self.admin.has_add_permission(self.factory), False)

    def test_get_actions_removes_delete_selected_method(self):
        """
        Actions should be: ['block_ips',
               'block_user_agents',
               'block_and_delete_ips',
               'block_and_delete_user_agents',
               'delete_queryset',
               ]
        """
        self.request.user = User.objects.create_superuser(username="admin", password="admin", email="a@a.com")

        self.assertIs("delete_selected" not in self.admin.get_actions(self.request), True)

    def test_get_actions_methods(self):
        """
        Actions should be: ['block_ips',
               'block_user_agents',
               'block_and_delete_ips',
               'block_and_delete_user_agents',
               'delete_queryset',
               ]
        """
        self.request.user = AnonymousUser()
        actions = [
            "block_ips",
            "block_user_agents",
            "block_and_delete_ips",
            "block_and_delete_user_agents",
            "delete_queryset",
        ]

        self.assertEqual(actions, list(self.admin.get_actions(self.request).keys()))

    def test_block_ips_single(self):
        qs = Hit.objects.filter(ip="127.0.0.5")

        self.admin.block_ips(self.request, qs)

        ip = BlockedIP.objects.last()

        self.assertEqual(ip.ip, "127.0.0.5")
        self.assertEqual(BlockedIP.objects.all().count(), 1)

    def test_block_ips_multiple(self):
        """
        Test adding `block_ips` via Admin action with multiple items.
        """
        qs = Hit.objects.all()[:5]

        self.admin.block_ips(self.request, qs)
        ips = BlockedIP.objects.values_list("ip", flat=True)

        self.assertEqual(ips[4], "127.0.0.5")
        self.assertEqual(BlockedIP.objects.all().count(), 5)

    def test_block_ips_add_only_once(self):
        """
        Test adding `block_ips` to ensure adding the same IP address more
        than once does not duplicate a record in the BlockedIP table.
        """
        ip = "127.0.0.5"

        qs = Hit.objects.filter(ip=ip)
        self.admin.block_ips(self.request, qs)

        self.assertEqual(BlockedIP.objects.all().count(), 1)

        # adding a second time should not increase the list
        qs = Hit.objects.filter(ip="127.0.0.5")
        self.admin.block_ips(self.request, qs)

        self.assertEqual(BlockedIP.objects.all().count(), 1)

    def test_block_user_agents_single(self):
        qs = Hit.objects.filter(ip="127.0.0.5")

        self.admin.block_user_agents(self.request, qs)
        ua = BlockedUserAgent.objects.last()

        self.assertEqual(ua.user_agent, "agent_5")
        self.assertEqual(BlockedUserAgent.objects.all().count(), 1)

    def test_block_user_agents_multiple(self):
        qs = Hit.objects.all()[:5]

        self.admin.block_user_agents(self.request, qs)
        uas = BlockedUserAgent.objects.values_list("user_agent", flat=True)

        self.assertEqual(uas[2], "agent_7")
        self.assertEqual(BlockedUserAgent.objects.all().count(), 5)

    def test_block_user_agents_add_only_once(self):
        """
        Test adding `block_ips` to ensure adding the same user agent more
        than once does not duplicate a record in the BlockedUserAgent table.
        """
        ip = "127.0.0.5"
        qs = Hit.objects.filter(ip=ip)

        self.admin.block_user_agents(self.request, qs)

        self.assertEqual(BlockedUserAgent.objects.all().count(), 1)

        # adding a second time should not increase the list
        qs = Hit.objects.filter(ip=ip)

        self.admin.block_user_agents(self.request, qs)

        self.assertEqual(BlockedUserAgent.objects.all().count(), 1)

    def test_delete_queryset(self):
        my_admin = User.objects.create_superuser("myuser", "myemail@example.com", "1234")
        self.request.user = my_admin

        qs = Hit.objects.all()[:5]
        self.admin.delete_queryset(self.request, qs)
        hit_count = HitCount.objects.last()

        self.assertEqual(Hit.objects.all().count(), 5)
        self.assertEqual(hit_count.hits, 5)

    def test_delete_queryset_single_item(self):
        my_admin = User.objects.create_superuser("myuser", "myemail@example.com", "1234")
        self.request.user = my_admin

        qs = Hit.objects.filter(ip="127.0.0.5")
        self.admin.delete_queryset(self.request, qs)
        hit_count = HitCount.objects.last()

        self.assertEqual(Hit.objects.all().count(), 9)
        self.assertEqual(hit_count.hits, 9)

    def test_delete_queryset_permission_denied(self):
        my_admin = User.objects.create_user("myuser", "myemail@example.com", "1234")
        self.request.user = my_admin

        qs = Hit.objects.all()[:5]
        with self.assertRaises(PermissionDenied):
            self.admin.delete_queryset(self.request, qs)

    def test_block_and_delete_ips(self):
        my_admin = User.objects.create_superuser("myuser", "myemail@example.com", "1234")
        self.request.user = my_admin

        qs = Hit.objects.all()[:5]
        self.admin.block_and_delete_ips(self.request, qs)
        hit_count = HitCount.objects.last()

        self.assertEqual(Hit.objects.all().count(), 5)
        self.assertEqual(hit_count.hits, 5)
        self.assertEqual(BlockedIP.objects.all().count(), 5)

    def test_block_and_delete_user_agents(self):
        my_admin = User.objects.create_superuser("myuser", "myemail@example.com", "1234")
        self.request.user = my_admin

        qs = Hit.objects.all()[:5]
        self.admin.block_and_delete_user_agents(self.request, qs)
        hit_count = HitCount.objects.last()

        self.assertEqual(Hit.objects.all().count(), 5)
        self.assertEqual(hit_count.hits, 5)
        self.assertEqual(BlockedUserAgent.objects.all().count(), 5)



import os
import sys


BASE_DIR = os.path.dirname(os.path.dirname(__file__))

sys.path.insert(0, os.path.dirname(__file__))
# Quick-start development settings - unsuitable for production
# See https://docs.djangoproject.com/en/1.6/howto/deployment/checklist/

# SECURITY WARNING: keep the secret key used in production secret!
SECRET_KEY = "t=d4ulc7kg99v4$d#s4ann+#&va0b2-npkq!qh19!l#_29@&$n"

# SECURITY WARNING: don't run with debug turned on in production!
DEBUG = True

ALLOWED_HOSTS = []


# Application definition

INSTALLED_APPS = (
    "blog",  # app to show hits
    "django.contrib.admin",
    "django.contrib.auth",
    "django.contrib.contenttypes",
    "django.contrib.sessions",
    "django.contrib.messages",
    "django.contrib.staticfiles",
    "hitcount",
)

MIDDLEWARE = (
    "django.contrib.sessions.middleware.SessionMiddleware",
    "django.middleware.common.CommonMiddleware",
    "django.middleware.csrf.CsrfViewMiddleware",
    "django.contrib.auth.middleware.AuthenticationMiddleware",
    "django.contrib.messages.middleware.MessageMiddleware",
    "django.middleware.clickjacking.XFrameOptionsMiddleware",
)

ROOT_URLCONF = "tests.urls"

# Database
# https://docs.djangoproject.com/en/1.6/ref/settings/#databases

DATABASES = {
    "default": {
        "ENGINE": "django.db.backends.sqlite3",
        "NAME": os.path.join(BASE_DIR, "db.sqlite3"),
    }
}

# this reduces the testing time appreciably
PASSWORD_HASHERS = ["django.contrib.auth.hashers.MD5PasswordHasher"]

TEMPLATES = [
    {
        "BACKEND": "django.template.backends.django.DjangoTemplates",
        "APP_DIRS": True,
        "OPTIONS": {
            # See: https://docs.djangoproject.com/en/dev/ref/settings/#template-debug
            "debug": DEBUG,
            # See: https://docs.djangoproject.com/en/dev/ref/settings/#template-context-processors
            "context_processors": [
                "django.template.context_processors.debug",
                "django.template.context_processors.request",
                "django.contrib.auth.context_processors.auth",
                "django.template.context_processors.i18n",
                "django.template.context_processors.media",
                "django.template.context_processors.static",
                "django.template.context_processors.tz",
                "django.contrib.messages.context_processors.messages",
                # Your stuff: custom template context processors go here
            ],
        },
    },
]

# Internationalization
# https://docs.djangoproject.com/en/1.6/topics/i18n/

LANGUAGE_CODE = "en-us"

TIME_ZONE = "UTC"

USE_I18N = True

USE_TZ = True

# Static files (CSS, JavaScript, Images)
# https://docs.djangoproject.com/en/1.6/howto/static-files/

STATIC_URL = "/static/"

DEFAULT_AUTO_FIELD = "django.db.models.AutoField"

# DJANGO HITCOUNT SPECIFIC VARIABLES

HITCOUNT_KEEP_HIT_ACTIVE = {"minutes": 60}
HITCOUNT_HITS_PER_IP_LIMIT = 0  # unlimited
HITCOUNT_HITS_PER_SESSION_LIMIT = 0  # unlimited.
HITCOUNT_EXCLUDE_USER_GROUP = ()  # not used
HITCOUNT_KEEP_HIT_IN_DATABASE = {"seconds": 10}



from collections import namedtuple

from django.contrib.auth.models import ContentType
from django.http import Http404

from hitcount.conf import settings
from hitcount.models import BlockedIP
from hitcount.models import BlockedUserAgent
from hitcount.models import Hit
from hitcount.utils import get_hitcount_model
from hitcount.utils import get_ip


class AJAXRequiredMixin:
    def dispatch(self, request, *args, **kwargs):
        if not request.META.get("HTTP_X_REQUESTED_WITH") == "XMLHttpRequest":
            raise Http404()

        return super().dispatch(request, *args, **kwargs)


class HitCountModelMixin:
    """
    HitCountMixin provides an easy way to add a `hit_count` property to your
    model that will return the related HitCount object.
    """

    @property
    def hit_count(self):
        ctype = ContentType.objects.get_for_model(self.__class__)
        HitCount = get_hitcount_model()
        hit_count, __ = HitCount.objects.get_or_create(content_type=ctype, object_pk=self.pk)
        return hit_count


class HitCountViewMixin:
    """
    Mixin to evaluate a HttpRequest and a HitCount and determine whether or not
    the HitCount should be incremented and the Hit recorded.
    """

    @staticmethod
    def hit_count(request, hitcount):
        """
        Called with a HttpRequest and HitCount object it will return a
        namedtuple:

        UpdateHitCountResponse(hit_counted=Boolean, hit_message='Message').

        `hit_counted` will be True if the hit was counted and False if it was
        not.  `'hit_message` will indicate by what means the Hit was either
        counted or ignored.
        """
        UpdateHitCountResponse = namedtuple("UpdateHitCountResponse", "hit_counted hit_message")
        # as of Django 1.8.4 empty sessions are not being saved
        # https://code.djangoproject.com/ticket/25489
        if not request.session.session_key:
            request.session.create()

        user = request.user

        if settings.HITCOUNT_USE_IP:
            ip = get_ip(request)
        else:
            ip = None

        user_agent = request.META.get("HTTP_USER_AGENT", "")[:255]

        # first, check our request against the IP blocked
        if BlockedIP.objects.is_blocked(ip):
            return UpdateHitCountResponse(False, "Not counted: user IP has been blocked")

        # second, check our request against the user agent blocked
        if BlockedUserAgent.objects.is_blocked(user_agent):
            return UpdateHitCountResponse(False, "Not counted: user agent has been blocked")

        # third, see if we are excluding a specific user group or not
        exclude_user_group = settings.HITCOUNT_EXCLUDE_USER_GROUP
        if exclude_user_group and request.user.is_authenticated:
            if request.user.groups.filter(name__in=exclude_user_group):
                return UpdateHitCountResponse(False, "Not counted: user group has been excluded")

        # eliminated first three possible exclusions, now on to checking our database of
        # active hits to see if we should count another one

        # check limit on hits from a unique ip address (HITCOUNT_HITS_PER_IP_LIMIT)
        if Hit.objects.has_limit_reached_by_ip(ip):
            return UpdateHitCountResponse(False, "Not counted: hits per IP address limit reached")

        session_key = request.session.session_key
        if Hit.objects.has_limit_reached_by_session(session_key, hitcount):
            return UpdateHitCountResponse(False, "Not counted: hits per session limit reached.")

        hit = Hit(
            session=session_key,
            hitcount=hitcount,
            ip=ip,
            user_agent=user_agent,
        )

        if request.user.is_authenticated:
            hit.user = user

            response = UpdateHitCountResponse(True, "Hit counted: user authentication")
        else:
            response = UpdateHitCountResponse(True, "Hit counted: session key")

        hit.save()
        return response



from django.http import HttpResponseBadRequest
from django.http import JsonResponse
from django.utils.translation import gettext_lazy as _
from django.views.generic import DetailView
from django.views.generic import View

from hitcount.mixins import AJAXRequiredMixin
from hitcount.mixins import HitCountViewMixin
from hitcount.utils import get_hitcount_model


HitCount = get_hitcount_model()


class HitCountJSONView(AJAXRequiredMixin, HitCountViewMixin, View):
    """
    JSON response view to handle HitCount POST.
    """

    def get(self, request, *args, **kwargs):
        msg = _("Hits counted via POST only.")
        return JsonResponse({"success": False, "error_message": msg})

    def post(self, request, *args, **kwargs):
        hitcount_pk = request.POST.get("hitcountPK")

        try:
            hitcount = HitCount.objects.get(pk=hitcount_pk)
        except HitCount.DoesNotExist:
            return HttpResponseBadRequest(_("HitCount object_pk not present."))

        hit_count_response = self.hit_count(request, hitcount)
        return JsonResponse(hit_count_response._asdict())


class HitCountDetailView(DetailView, HitCountViewMixin):
    """
    HitCountDetailView provides an inherited DetailView that will inject the
    template context with a `hitcount` variable giving you the number of
    Hits for an object without using a template tag.

    Optionally, by setting `count_hit = True` you can also do the business of
    counting the Hit for this object (in lieu of using JavaScript).  It will
    then further inject the response from the attempt to count the Hit into
    the template context.
    """

    count_hit = False

    def get_context_data(self, **kwargs):
        context = super().get_context_data(**kwargs)

        assert self.object, "The object for Detail view has not been defined"

        hit_count = HitCount.objects.get_for_object(self.object)
        hits = hit_count.hits
        context["hitcount"] = {"pk": hit_count.pk}

        if self.count_hit:
            hit_count_response = self.hit_count(self.request, hit_count)
            if hit_count_response.hit_counted:
                hits = hits + 1
            context["hitcount"]["hit_counted"] = hit_count_response.hit_counted
            context["hitcount"]["hit_message"] = hit_count_response.hit_message

        context["hitcount"]["total_hits"] = hits

        return context



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.views.decorators.csrf import ensure_csrf_cookie
from django.views.generic import DetailView
from django.views.generic import TemplateView

from blog.models import Post
from hitcount.views import HitCountDetailView


class PostMixinDetailView(object):
    """
    Mixin to same us some typing.  Adds context for us!
    """

    model = Post

    def get_context_data(self, **kwargs):
        context = super(PostMixinDetailView, self).get_context_data(**kwargs)
        context["post_list"] = Post.objects.all()[:5]
        context["post_views"] = ["ajax", "detail", "detail-with-count"]
        return context


class IndexView(PostMixinDetailView, TemplateView):
    template_name = "blog/index.html"


class PostDetailJSONView(PostMixinDetailView, DetailView):
    template_name = "blog/post_ajax.html"

    @classmethod
    def as_view(cls, **initkwargs):
        view = super(PostDetailJSONView, cls).as_view(**initkwargs)
        return ensure_csrf_cookie(view)


class PostDetailView(PostMixinDetailView, HitCountDetailView):
    """
    Generic hitcount class based view.
    """

    pass


class PostCountHitDetailView(PostMixinDetailView, HitCountDetailView):
    """
    Generic hitcount class based view that will also perform the hitcount logic.
    """

    count_hit = True



# Generated by Django 3.2.4 on 2021-06-16 20:26
from django.db import migrations
from django.db import models


class Migration(migrations.Migration):

    dependencies = [
        ('hitcount', '0004_auto_20200704_0933'),
    ]

    operations = [
        migrations.RenameModel(
            old_name='BlacklistIP',
            new_name='BlockedIP',
        ),
        migrations.RenameModel(
            old_name='BlacklistUserAgent',
            new_name='BlockedUserAgent',
        ),
        migrations.AlterModelOptions(
            name='blockedip',
            options={'verbose_name': 'Blocked IP', 'verbose_name_plural': 'Blocked IPs'},
        ),
        migrations.AlterModelOptions(
            name='blockeduseragent',
            options={'verbose_name': 'Blocked User Agent', 'verbose_name_plural': 'Blocked User Agents'},
        ),
        migrations.AlterField(
            model_name='hit',
            name='ip',
            field=models.CharField(db_index=True, editable=False, max_length=40, null=True),
        ),
        migrations.AlterModelTable(
            name='blockedip',
            table=None,
        ),
        migrations.AlterModelTable(
            name='blockeduseragent',
            table=None,
        ),
    ]



# -*- coding: utf-8 -*-
from django.contrib import admin
from django.urls import include
from django.urls import path

from blog import views

admin.autodiscover()

urlpatterns = [
    path("", views.IndexView.as_view(), name="index"),
    path("generic-detail-view-ajax/<int:pk>/", views.PostDetailJSONView.as_view(), name="ajax"),
    path("hitcount-detail-view/<int:pk>/", views.PostDetailView.as_view(), name="detail"),
    path("hitcount-detail-view-count-hit/<int:pk>/", views.PostCountHitDetailView.as_view(), name="detail-with-count"),
    # for our built-in ajax post view
    path("hitcount/", include("hitcount.urls", namespace="hitcount")),
]

urlpatterns.append(path("admin/", admin.site.urls))



import django
from django.conf import settings as django_settings
from django.utils.functional import LazyObject

from hitcount.conf import defaults as app_settings


class LazySettings(LazyObject):
    def _setup(self):
        self._wrapped = Settings(app_settings, django_settings)


_django_version = django.VERSION

_DEPRECATED_DJANGO_SETTINGS = {
    "DEFAULT_FILE_STORAGE" if (4, 2) <= _django_version < (5, 1) else None,
    "STATICFILES_STORAGE" if (4, 2) <= _django_version < (5, 1) else None,
    "USE_L10N" if (4, 0) <= _django_version < (5, 0) else None,
}


class Settings:
    def __init__(self, *args):
        [
            setattr(self, attr, getattr(item, attr))
            for item in args
            for attr in dir(item)
            if attr == attr.upper() and attr.upper() not in _DEPRECATED_DJANGO_SETTINGS
        ]


settings = LazySettings()



from datetime import timedelta
from unittest.mock import patch

from django.shortcuts import reverse
from django.template import Context
from django.template import Template
from django.template import TemplateSyntaxError
from django.test import TestCase
from django.utils import timezone

from blog.models import Post
from hitcount.models import Hit
from hitcount.utils import get_hitcount_model

HitCount = get_hitcount_model()


class BaseTemplateTagsTest(TestCase):
    fixtures = ["initial_data.json"]

    @classmethod
    def setUpTestData(cls):
        super().setUpTestData()
        cls.post = Post.objects.first()
        hit_count = HitCount.objects.create(content_object=cls.post)

        for x in range(10):
            created = timezone.now() - timedelta(minutes=x * 15)
            with patch("django.utils.timezone.now") as mock_now:
                mock_now.return_value = created

                Hit.objects.create(hitcount=hit_count)

    @staticmethod
    def _render(template, context):
        return Template(template).render(Context(context))


class TestGetHitCount(BaseTemplateTagsTest):
    def test_returns_0(self):
        """
        {% get_hit_count for post%}

        If no HitCount object exists, the template tag should return zero
        and have created an object for the next time.
        """
        # no HitCounts to start
        self.assertEqual(HitCount.objects.all().count(), 1)

        post2 = Post.objects.create(title="second", content="post!")

        out = Template("{% load hitcount_tags %}{% get_hit_count for post %}").render(Context({"post": post2}))

        # zero hits, but now one object
        self.assertEqual(str(0), out)
        self.assertEqual(HitCount.objects.all().count(), 2)

    def test_returns_10(self):
        """
        {% get_hit_count for post%}

        Test tag with multiple hits.
        """
        out = Template("{% load hitcount_tags %}{% get_hit_count for post %}").render(Context({"post": self.post}))

        self.assertEqual(str(10), out)

    def test_as_variable(self):
        """
        {% get_hit_count for post as hits %}

        Test tag with output as variable.
        """
        out = Template("{% load hitcount_tags %}{% get_hit_count for post as hits %}Total Hits: {{ hits }}").render(
            Context({"post": self.post})
        )

        self.assertEqual("Total Hits: 10", out)

    def test_within(self):
        """
        {% get_hit_count for [object] within [ex: "hours=1"] %}

        Test tag with multiple hits.
        """
        out = Template('{% load hitcount_tags %}{% get_hit_count for post within "hours=1" %}').render(
            Context({"post": self.post})
        )

        self.assertEqual(str(4), out)

    def test_within_multiple_time_args(self):
        """
        {% get_hit_count for [object] within [ex: "hours=1,minutes=30"] %}

        Test tag with multiple hits and multiple time args.
        """
        out = Template('{% load hitcount_tags %}{% get_hit_count for post within "hours=1,minutes=30" %}').render(
            Context({"post": self.post})
        )

        self.assertEqual(str(6), out)

    def test_within_multiple_time_args_without_quotes(self):
        """
        {% get_hit_count for [object] within [ex: hours=1,minutes=30] %}

        Test tag with multiple hits and multiple time args without quotes.
        """
        out = Template("{% load hitcount_tags %}{% get_hit_count for post within hours=1,minutes=30 %}").render(
            Context({"post": self.post})
        )

        self.assertEqual(str(6), out)

    def test_within_as_var(self):
        """
        {% get_hit_count for [object] within [ex: "hours=1"] as [var] %}

        Test tag with multiple hits, get as a variable.
        """
        out = Template(
            "{% load hitcount_tags %}"
            '{% get_hit_count for post within "hours=1" as hits %}'
            "Total Hits in last hour: {{ hits }}"
        ).render(Context({"post": self.post}))

        self.assertEqual("Total Hits in last hour: 4", out)

    def test_within_multiple_time_args_as_var(self):
        """
        {% get_hit_count for [object] within [ex: "hours=1,minutes=30"] as [var] %}

        Test tag with multiple hits and multiple time args as a variable.
        """
        out = Template(
            "{% load hitcount_tags %}"
            '{% get_hit_count for post within "hours=1,minutes=30" as hits %}'
            "Total Hits in last 1h 15m: {{ hits }}"
        ).render(Context({"post": self.post}))

        self.assertEqual("Total Hits in last 1h 15m: 6", out)

    def test_parsing_error_for_invalid_date_or_time(self):
        self.assertRaises(
            TemplateSyntaxError,
            self._render,
            '{% load hitcount_tags %}{% get_hit_count for post within "foo=1,bar=30" %}',
            {"post": self.post},
        )

    def test_parsing_error_for_non_existent_context_variable(self):
        self.assertRaises(
            TemplateSyntaxError,
            self._render,
            "{% load hitcount_tags %}{% get_hit_count for post %}",
            {"post_doesnt_context": self.post},
        )

    def test_parsing_error_for_wrong_number_of_args(self):
        self.assertRaises(
            TemplateSyntaxError, self._render, "{% load hitcount_tags %}{% get_hit_count post %}", {"post": self.post}
        )

    def test_parsing_error_for_passing_invalid_object(self):
        self.assertRaises(
            TemplateSyntaxError,
            self._render,
            "{% load hitcount_tags %}{% get_hit_count post %}",
            {"post": "bob the baker"},
        )


class TestGetHitCountJSVariables(BaseTemplateTagsTest):
    def test_usage(self):
        """
        {% get_hit_count_js_variables for [object] as [var_name] %}

        Test retrieval of javascript variables.
        """
        out = Template(
            "{% load hitcount_tags %}"
            "{% get_hit_count_js_variables for post as hit_count_js %}"
            "pk: {{ hit_count_js.pk }} || "
            "url: {{ hit_count_js.ajax_url }} || "
            "hits: {{ hit_count_js.hits }}"
        ).render(Context({"post": self.post}))

        pk = self.post.hit_count.pk
        self.assertEqual("pk: %s || url: /hitcount/hit/ajax/ || hits: 10" % pk, out)

    def test_parsing_errors_for_non_existent_context_variable(self):
        self.assertRaises(
            TemplateSyntaxError,
            self._render,
            "{% load hitcount_tags %}{% get_hit_count_js_variables as hit_count_js %}",
            {"post_variable_wrong": self.post},
        )

    def test_parsing_errors_for_wrong_number_of_variables(self):
        self.assertRaises(
            TemplateSyntaxError,
            self._render,
            "{% load hitcount_tags %}{% get_hit_count_js_variables as hit_count_js %}",
            {"post": self.post},
        )

    def test_parsing_errors_for_passing_wrong_object(self):
        self.assertRaises(
            TemplateSyntaxError,
            self._render,
            "{% load hitcount_tags %}{% get_hit_count_js_variables for post as hit_count_js %}",
            {"post": "bob the baker"},
        )


class TestInsertHitCountJSVariables(BaseTemplateTagsTest):
    def test_usage(self):
        """
        {% insert_hit_count_js_variables for [object] %}

        Test for writing of js variables directly onto the template.
        """
        out = Template("{% load hitcount_tags %}{% insert_hit_count_js_variables for post %}").render(
            Context({"post": self.post})
        )

        pk = self.post.hit_count.pk

        self.assertEqual(
            (
                '<script type="text/javascript">\n'
                "var hitcountJS = {hitcountPK : '%s',hitcountURL :"
                " '/hitcount/hit/ajax/'};\n</script>"
            )
            % pk,
            out,
        )

    def test_parsing_errors_for_wrong_number_of_variables(self):
        self.assertRaises(
            TemplateSyntaxError,
            self._render,
            "{% load hitcount_tags %}{% insert_hit_count_js_variables post %}",
            {"post": self.post},
        )

    def test_parsing_errors_for_non_existent_context_variable(self):
        self.assertRaises(
            TemplateSyntaxError,
            self._render,
            "{% load hitcount_tags %}{% insert_hit_count_js_variables for post %}",
            {"post_doesnt_context": self.post},
        )

    def test_parsing_errors_for_passing_wrong_object(self):
        self.assertRaises(
            TemplateSyntaxError,
            self._render,
            "{% load hitcount_tags %}{% insert_hit_count_js_variables for post %}",
            {"post": "bob the baker"},
        )


class TestInsertHitCountJS(BaseTemplateTagsTest):
    def test_without_debug(self):
        """
        {% insert_hit_count_js for [object] %}

        Test for writing of js directly onto the template.
        """
        out = Template("{% load hitcount_tags %}{% insert_hit_count_js for post %}").render(
            Context({"post": self.post})
        )

        pk = self.post.hit_count.pk
        expected = """
<script type="text/javascript">
//<![CDATA[
jQuery(document).ready(function($) {
  $.postCSRF("%(url)s", {
    hitcountPK: "%(pk)s"
  });
});
//]]>
</script>
"""

        self.assertEqual(
            expected % {"url": reverse("hitcount:hit_ajax"), "pk": pk},
            out,
        )

    def test_with_debug(self):
        """
        {% insert_hit_count_js for [object] debug=True %}

        Test for writing of js directly onto the template.
        """
        out = Template("{% load hitcount_tags %}{% insert_hit_count_js for post debug %}").render(
            Context({"post": self.post})
        )

        pk = self.post.hit_count.pk
        expected = """
<script type="text/javascript">
//<![CDATA[
jQuery(document).ready(function($) {
  $.postCSRF("%(url)s", {
    hitcountPK: "%(pk)s"
  }).done(function(data) {
    console.log(data);
  }).fail(function(data) {
    console.log('dj-hitcount: AJAX POST failed.');
    console.log(data);
  });
});
//]]>
</script>
"""

        self.assertEqual(
            expected % {"url": reverse("hitcount:hit_ajax"), "pk": pk},
            out,
        )

    def test_parsing_errors_for_wrong_number_of_variables(self):
        self.assertRaises(
            TemplateSyntaxError,
            self._render,
            "{% load hitcount_tags %}{% insert_hit_count_js post %}",
            {"post": self.post},
        )

    def test_parsing_errors_for_non_existent_context_variable(self):
        self.assertRaises(
            TemplateSyntaxError,
            self._render,
            "{% load hitcount_tags %}{% insert_hit_count_js for post %}",
            {"post_doesnt_context": self.post},
        )

    def test_parsing_errors_for_passing_wrong_object(self):
        self.assertRaises(
            TemplateSyntaxError,
            self._render,
            "{% load hitcount_tags %}{% insert_hit_count_js for post %}",
            {"post": "bob the baker"},
        )



HITCOUNT_KEEP_HIT_ACTIVE = {"days": 7}

HITCOUNT_HITS_PER_IP_LIMIT = 0

HITCOUNT_HITS_PER_SESSION_LIMIT = 0

HITCOUNT_EXCLUDE_USER_GROUP = ()

HITCOUNT_KEEP_HIT_IN_DATABASE = {"days": 30}

HITCOUNT_USE_IP = True

HITCOUNT_HITCOUNT_MODEL = "hitcount.HitCount"



from django.test import TestCase

from hitcount.models import BlockedIP
from hitcount.models import BlockedUserAgent


class TestBlockIPManager(TestCase):
    def test_is_blocked(self):
        # without IP.
        self.assertIs(BlockedIP.objects.is_blocked(), False)

        BlockedIP.objects.create(ip="10.1.2.1")

        self.assertIs(BlockedIP.objects.is_blocked("10.1.2.1"), True)
        self.assertIs(BlockedIP.objects.is_blocked("10.1.1.1"), False)


class TestBlockUserAgentManager(TestCase):
    def test_is_blocked(self):
        # without an agent.
        self.assertIs(BlockedUserAgent.objects.is_blocked(), False)

        user_agent_windows = "Mozilla/5.0 (Windows NT 6.1; Win64; x64; rv:47.0) Gecko/20100101 Firefox/47.0"
        user_agent_mac = "Mozilla/5.0 (Macintosh; Intel Mac OS X x.y; rv:42.0) Gecko/20100101 Firefox/42.0"

        BlockedUserAgent.objects.create(user_agent=user_agent_windows)

        self.assertIs(BlockedUserAgent.objects.is_blocked(user_agent_windows), True)
        self.assertIs(BlockedUserAgent.objects.is_blocked(user_agent_mac), False)



from datetime import timedelta

from django.core.management import BaseCommand
from django.utils import timezone

from hitcount.conf import settings
from hitcount.models import Hit


class Command(BaseCommand):
    help = "Can be run as a cronjob or directly to clean out old Hits objects from the database."

    def __init__(self, *args, **kwargs):
        super(Command, self).__init__(*args, **kwargs)

    def handle(self, *args, **kwargs):
        self.handle_noargs()

    def handle_noargs(self, **options):
        grace = settings.HITCOUNT_KEEP_HIT_IN_DATABASE
        period = timezone.now() - timedelta(**grace)
        qs = Hit.objects.filter(created__lt=period)
        number_removed = qs.count()
        qs.delete()
        self.stdout.write("Successfully removed %s Hits" % number_removed)
